@@ -12,6 +12,7 @@ from dataquality.exceptions import GalileoException, GalileoWarning
 from dataquality.schemas.hf import HFCol
 from dataquality.schemas.ner import TaggingSchema
 from dataquality.schemas.split import conform_split
+from dataquality.utils.helpers import check_noop
 from dataquality.utils.hf_tokenizer import LabelTokenizer
 
 
@@ -90,9 +91,11 @@ def _validate_dataset(dd: DatasetDict) -> DatasetDict:
         )
     for key in dd.keys():
         ds = dd[key]
+        # Filter out the samples with no tokens
+        ds = ds.filter(lambda row: len(row[HFCol.tokens]) != 0)
         if HFCol.ner_tags not in ds.features:
             if HFCol.tags in ds.features:
-                dd[key] = ds.rename_column(HFCol.tags, HFCol.ner_tags)
+                ds = ds.rename_column(HFCol.tags, HFCol.ner_tags)
                 warnings.warn(
                     f"{HFCol.tags} column found, it will be "
                     f"renamed to {HFCol.ner_tags}",
@@ -101,10 +104,11 @@ def _validate_dataset(dd: DatasetDict) -> DatasetDict:
             else:
                 raise GalileoException("Each dataset must have either ner_tags or tags")
         if HFCol.id in ds.features:
-            dd[key] = ds.remove_columns(HFCol.id)
+            ds = ds.remove_columns(HFCol.id)
             warnings.warn(
                 f"{HFCol.id} column found, it will be replaced", GalileoWarning
             )
+        dd[key] = ds
     return dd
 
 
@@ -128,10 +132,12 @@ def _extract_labels_from_ds(dd: DatasetDict) -> List[str]:
     )
 
 
+@check_noop
 def tokenize_and_log_dataset(
     dd: DatasetDict,
     tokenizer: PreTrainedTokenizerBase,
     label_names: Optional[List[str]] = None,
+    meta: Optional[List[str]] = None,
 ) -> DatasetDict:
     """This function tokenizes a huggingface DatasetDict and aligns the labels to BPE
 
@@ -140,7 +146,14 @@ def tokenize_and_log_dataset(
 
     :param dd: DatasetDict from huggingface to log
     :param tokenizer: The pretrained tokenizer from huggingface
+    :param label_names: Optional list of labels for the dataset. These can typically
+        be extracted automatically (if the dataset came from hf datasets hub or was
+        exported via Galileo dataquality). If they cannot be extracted, an error will
+        be raised requesting label names
+    :param meta: Optional metadata columns to be logged. The columns must be present
+        in at least one of the splits of the dataset.
     """
+    meta = meta or []
     dd = _validate_dataset(dd)
     if label_names is not None and len(label_names):
         assert isinstance(
@@ -166,7 +179,8 @@ def tokenize_and_log_dataset(
         ids = list(range(len(dataset)))
         dataset = dataset.add_column(HFCol.id, ids)
         tokenized_dataset[split] = dataset
-        dq.log_dataset(dataset, split=dq_split)  # type: ignore
+        split_meta = [c for c in meta if c in dataset.features]
+        dq.log_dataset(dataset, split=dq_split, meta=split_meta)  # type: ignore
     return tokenized_dataset
 
 
