@@ -1,9 +1,12 @@
 import os
 import warnings
-from typing import Dict, List, Optional, Union
+from typing import Any, Dict, List, Optional, Tuple, Union
 from uuid import uuid4
 
+import pandas as pd
 import vaex
+from cachetools import LRUCache, cached
+from cachetools.keys import hashkey
 from vaex.dataframe import DataFrame
 
 from dataquality.clients.api import ApiClient
@@ -20,6 +23,37 @@ api_client = ApiClient()
 object_store = ObjectStore()
 
 
+def _cache_key(*args: Tuple, **kwargs: Dict[str, Any]) -> Tuple:
+    """Custom cache key that includes the updated_at timestamp for a run
+
+    https://cachetools.readthedocs.io/en/latest/#cachetools.keys.typedkey
+
+    We use this cache only for the heavy functions in metrics (downloading things
+    from the server). We have a modified cache key that includes the last time
+    this run was updated, in case that has changed since the last call.
+
+    NOTE: It is assumed this cache takes the project_name and run_name as either args
+    0 and 1, or 1 and 2 (see comments), never kwargs. As such, this cache is meant to be
+    used for internal (not user facing) functions.
+    """
+    # First 2 arguments are project and run name
+    if isinstance(args[0], str):
+        project_name, run_name = args[0], args[1]
+    # First argument is the dataframe, then project and run name
+    else:
+        project_name, run_name = args[1], args[2]
+    updated_ts = api_client.get_project_run_by_name(
+        str(project_name), str(run_name)
+    ).get("updated_at")
+    key = hashkey(*args, *kwargs.items())
+    key += (updated_ts,)
+    return key
+
+
+def _get_cache() -> LRUCache:
+    return LRUCache(maxsize=128)
+
+
 def create_edit(
     project_name: str,
     run_name: str,
@@ -183,6 +217,39 @@ def display_distribution(
     fig.show()
 
 
+@cached(_get_cache(), key=_cache_key)
+def _download_df(
+    project_name: str,
+    run_name: str,
+    split: Split,
+    inference_name: str,
+    file_type: FileType,
+    hf_format: bool,
+    tagging_schema: Optional[TaggingSchema],
+    filter_params: FilterParams,
+) -> DataFrame:
+    """Helper function to download the dataframe to take advantage of caching
+
+    The filter_params class FilterParams is hashable (wherein a dict is not) so we
+    force passing that in `get_dataframe`
+    """
+    split = conform_split(split)
+
+    file_name = f"/tmp/{uuid4()}-data.{file_type}"
+    api_client.export_run(
+        project_name,
+        run_name,
+        split,
+        inference_name=inference_name,
+        file_name=file_name,
+        filter_params=filter_params.dict(),
+        hf_format=hf_format,
+        tagging_schema=tagging_schema,
+    )
+    data_df = vaex.open(file_name)
+    return data_df
+
+
 def get_dataframe(
     project_name: str,
     run_name: str,
@@ -195,7 +262,8 @@ def get_dataframe(
     hf_format: bool = False,
     tagging_schema: Optional[TaggingSchema] = None,
     filter: Union[FilterParams, Dict] = None,
-) -> DataFrame:
+    as_pandas: bool = True,
+) -> Union[pd.DataFrame, DataFrame]:
     """Gets the dataframe for a run/split
 
     Downloads an arrow (or specified type) file to your machine and returns a loaded
@@ -223,24 +291,27 @@ def get_dataframe(
         If hf_format is True, you must pass a tagging schema
     :param filter: Optional filter to provide to restrict the distribution to only to
         matching rows. See `dq.schemas.metrics.FilterParams`
+    :param as_pandas: Whether to return the dataframe as a pandas df (or vaex if False)
+        If you are having memory issues (the data is too large), set this to False,
+        and vaex will memory map the data. If any columns returned are multi-dimensional
+        (embeddings, probabilities etc), vaex will always be returned, because pandas
+        cannot support multi-dimensional columns. Default True
     """
     split = conform_split(split)
     project_id, run_id = api_client._get_project_run_id(project_name, run_name)
     task_type = api_client.get_task_type(project_id, run_id)
+    filter_params = FilterParams(**_validate_filter(filter))
 
-    file_name = f"/tmp/{uuid4()}-data.{file_type}"
-    filter_params = _validate_filter(filter)
-    api_client.export_run(
+    data_df = _download_df(
         project_name,
         run_name,
         split,
-        inference_name=inference_name,
-        file_name=file_name,
-        filter_params=filter_params,
-        hf_format=hf_format,
-        tagging_schema=tagging_schema,
+        inference_name,
+        file_type,
+        hf_format,
+        tagging_schema,
+        filter_params,
     )
-    data_df = vaex.open(file_name)
     return _process_exported_dataframe(
         data_df,
         project_name,
@@ -252,6 +323,7 @@ def get_dataframe(
         include_probs,
         include_token_indices,
         hf_format,
+        as_pandas,
     )
 
 
@@ -266,11 +338,12 @@ def get_edited_dataframe(
     include_token_indices: bool = False,
     hf_format: bool = False,
     tagging_schema: Optional[TaggingSchema] = None,
-) -> DataFrame:
+    as_pandas: bool = True,
+) -> Union[pd.DataFrame, DataFrame]:
     """Gets the edited dataframe for a run/split
 
     Exports a run/split's data with all active edits in the edits cart and returns
-    a vaex dataframe
+    a vaex or pandas dataframe
 
     Special note for NER. By default, the data will be downloaded at a sample level
     (1 row per sample text), with spans for each sample in a `spans` column in a
@@ -292,6 +365,11 @@ def get_edited_dataframe(
         Whether to export the dataframe in a HuggingFace compatible format
     :param tagging_schema: (NER only)
         If hf_format is True, you must pass a tagging schema
+    :param as_pandas: Whether to return the dataframe as a pandas df (or vaex if False)
+        If you are having memory issues (the data is too large), set this to False,
+        and vaex will memory map the data. If any columns returned are multi-dimensional
+        (embeddings, probabilities etc), vaex will always be returned, because pandas
+        cannot support multi-dimensional columns. Default True
     """
     split = conform_split(split)
     project_id, run_id = api_client._get_project_run_id(project_name, run_name)
@@ -319,9 +397,11 @@ def get_edited_dataframe(
         include_probs,
         include_token_indices,
         hf_format,
+        as_pandas,
     )
 
 
+@cached(_get_cache(), key=_cache_key)
 def _process_exported_dataframe(
     data_df: DataFrame,
     project_name: str,
@@ -333,7 +413,8 @@ def _process_exported_dataframe(
     include_probs: bool = False,
     include_token_indices: bool = False,
     hf_format: bool = False,
-) -> DataFrame:
+    as_pandas: bool = True,
+) -> Union[pd.DataFrame, DataFrame]:
     """Process dataframe after export of run or edits.
 
     See `get_dataframe` and `get_edited_dataframe` for details"""
@@ -399,6 +480,12 @@ def _process_exported_dataframe(
             if "sample_id" in data_df.get_column_names():
                 data_df.rename("sample_id", "id")
             data_df = data_df.join(raw_tokens, on="id")
+    if as_pandas:
+        # If any columns are multi-dimensional (embeddings, probs etc), must return vaex
+        for col in data_df.get_column_names():
+            if data_df[col].ndim > 1:
+                return data_df
+        return data_df.to_pandas_df()
     return data_df
 
 
@@ -523,6 +610,7 @@ def get_tasks_for_run(project_name: str, run_name: str) -> List[str]:
     return api_client.get_tasks_for_run(project_name, run_name)
 
 
+@cached(_get_cache(), key=_cache_key)
 def _get_hdf5_file_for_epoch(
     project_name: str,
     run_name: str,
